/* Copyright (C) 2024 Hulubei National Institute of Physics and Nuclear Engineering - Horia Hulubei, Bucharest
   SPDX-License-Identifier: GPL-3.0-only
   Authors: Alexandru Bercuci [committer] */

// --------------------------------------------------------------------------
//
// Macro for simulation & reconstruction QA
//
// The following naming conventions are assumed:
// Raw data file:  [dataset].event.raw.root
// Transport file: [dataset].tra.root
// Parameter file:  [dataset].par.root
// Reconstruction file: [dataset].rec.root
//
// S. Gorbunov 28/09/2020
//
// --------------------------------------------------------------------------

// Includes needed for IDE
#if !defined(__CLING__)
#include "CbmDefs.h"
#include "CbmMCDataManager.h"
#include "CbmMuchTransportQa.h"
#include "CbmQaManager.h"
#include "CbmSetup.h"

#include <FairFileSource.h>
#include <FairMonitor.h>
#include <FairParAsciiFileIo.h>
#include <FairParRootFileIo.h>
#include <FairRootFileSink.h>
#include <FairRunAna.h>
#include <FairRuntimeDb.h>
#include <FairSystemInfo.h>

#include <TStopwatch.h>
#endif

void run_recoQa(Int_t nEvents = -1, TString recFile = "2391_node8_0_0000.rec.root",
                TString setupName = "mcbm_beam_2022_05_23_nickel", Bool_t bUseMC = kFALSE)
{

  // ========================================================================
  //          Adjust this part according to your requirements

  // -----   Logger settings   ----------------------------------------------
  FairLogger::GetLogger()->SetLogScreenLevel("INFO");
  FairLogger::GetLogger()->SetLogVerbosityLevel("LOW");
  // ------------------------------------------------------------------------

  // -----   Environment   --------------------------------------------------
  int verbose    = 6;                              // verbose level
  TString myName = "recoQA";                       // this macro's name for screen output
  TString srcDir = gSystem->Getenv("VMCWORKDIR");  // top source directory
  // ------------------------------------------------------------------------

  // -----   In- and output file names   ------------------------------------
  TString dataset = recFile;
  dataset.ReplaceAll(".rec.root", "");
  TString traFile = dataset + ".tra.root";
  //TString rawFile  = dataset + ".event.raw.root";
  TString parFile  = dataset + ".par.root";
  TString sinkFile = dataset + ".rqa.root";
  TString geoFile  = setupName + ".geo.root";

  // ------------------------------------------------------------------------

  // -----   Load the geometry setup   -------------------------------------
  std::cout << '\n';

  CbmSetup* setup = CbmSetup::Instance();
  setup->LoadSetup(setupName);
  //setup->SetActive(ECbmModuleId::kSts, false);
  //setup->SetActive(ECbmModuleId::kTof, false);

  // In general, the following parts need not be touched
  // ========================================================================

  // -----   Timer   --------------------------------------------------------
  TStopwatch timer;
  timer.Start();
  // ------------------------------------------------------------------------

  // ----    Debug option   -------------------------------------------------
  gDebug = 0;
  // ------------------------------------------------------------------------

  std::map<std::string, TGeoHMatrix>* matrices{nullptr};
  TFile* misalignmentMatrixRootfile = TFile::Open(Form("AlignmentMatrices_%s.root", setupName.Data()), "READ");
  if (misalignmentMatrixRootfile && misalignmentMatrixRootfile->IsOpen()) {
    gDirectory->GetObject("MisalignMatrices", matrices);
    misalignmentMatrixRootfile->Close();
  }
  else
    LOG(warning) << "Could not find usable alignment file. Skip.";

  // -----   FairRunAna   ---------------------------------------------------
  FairFileSource* inputSource = new FairFileSource(recFile);
  if (bUseMC) {
    inputSource->AddFriend(traFile);
  }
  //inputSource->AddFriend(rawFile);

  FairRunAna* run = new FairRunAna();
  run->SetSource(inputSource);
  run->SetGenerateRunInfo(kFALSE);
  run->SetGeomFile(geoFile);

  FairRootFileSink* sink = new FairRootFileSink(sinkFile);
  run->SetSink(sink);

  TString monitorFile{sinkFile};
  monitorFile.ReplaceAll("rqa", "rqa.moni");
  FairMonitor::GetMonitor()->EnableMonitor(kTRUE, monitorFile);
  // ------------------------------------------------------------------------

  if (matrices) run->AddAlignmentMatrices(*matrices);

  // -----   MCDataManager (legacy mode)  -----------------------------------
  if (bUseMC) {
    std::cout << "-I- " << myName << ": Adding MC manager and MC to reco matching tasks\n";

    auto* mcManager = new CbmMCDataManager("MCDataManager", 1);
    mcManager->AddFile(traFile);
    run->AddTask(mcManager);

    auto* matchRecoToMC = new CbmMatchRecoToMC();
    // NOTE: Matching is suppressed, if there are hit and cluster matches already in the tree. If there
    //       are no hit matches, they are produced on this stage.
    matchRecoToMC->SuppressHitReMatching();
    run->AddTask(matchRecoToMC);
  }
  // ------------------------------------------------------------------------

  // ----- Tracking detector interface --------------------------------------
  run->AddTask(new CbmTrackingDetectorInterfaceInit());
  // ------------------------------------------------------------------------


  // ------------------------------------------------------------------------

  // TODO: read tracking parameters from the file like CaOutputQa does
  //TODO:  instead of initializing the tracker
  //TString caParFile = recFile;
  //caParFile.ReplaceAll(".root", ".ca.par");
  //auto* pCaOutputQa = new cbm::ca::OutputQa(verbose, bUseMC);
  //pCaOutputQa->ReadParameters(caParFile.Data());

  // L1 CA track finder setup
  auto l1 = new CbmL1("CA");
  l1->SetMcbmMode();

  // User configuration example for CA:
  //l1->SetConfigUser(srcDir + "/macro/L1/configs/ca_params_user_example.yaml");
  run->AddTask(l1);

  // ----- Reco QA --------------------------------------------
  CbmRecoQaTask::Detector* det(nullptr);
  CbmRecoQaTask::Detector::View* view(nullptr);
  CbmRecoQaTask* recoQa = new CbmRecoQaTask();
  // recoQa->SetSetupClass(CbmRecoQaTask::kMcbm22);
  recoQa->SetSetupClass(CbmRecoQaTask::kMcbm24);
  recoQa->AddEventFilter(CbmRecoQaTask::EventFilter::eEventCut::kMultTrk)->SetFilter({1, 2});
  recoQa->AddTrackFilter(CbmRecoQaTask::TrackFilter::eTrackCut::kSts)->SetFilter({3});
  recoQa->AddTrackFilter(CbmRecoQaTask::TrackFilter::eTrackCut::kTrd)->SetFilter({2});
  recoQa->AddTrackFilter(CbmRecoQaTask::TrackFilter::eTrackCut::kTof)->SetFilter({1});
  //   if ((det = recoQa->GetDetector(ECbmModuleId:kSts))) {
  //     view = det->GetView("U0L0M0");
  //     view->SetProjection();
  //   }
  //   double cos25 = TMath::Cos(TMath::DegToRad() * 25.);
  //   recoQa->SetProjections({15.1 * cos25, 0., -20 * cos25, -38 * cos25, -50.5 * cos25});
  run->AddTask(recoQa);
  run->SetGenerateRunInfo(kFALSE);

  // ------------------------------------------------------------------------
  //-----  Load Parameters --------------------------------------------------
  std::cout << std::endl << std::endl;
  std::cout << "-I- " << myName << ": Set runtime DB" << std::endl;
  FairRuntimeDb* rtdb       = run->GetRuntimeDb();
  FairParRootFileIo* parIo1 = new FairParRootFileIo();
  parIo1->open(parFile.Data(), "in");
  rtdb->setFirstInput(parIo1);

  // ------------------------------------------------------------------------
  // -----   Run initialisation   -------------------------------------------
  std::cout << std::endl;
  std::cout << "-I- " << myName << ": Initialise run" << std::endl;
  run->Init();

  rtdb->print();

  // -----   Start run   ----------------------------------------------------
  std::cout << std::endl << std::endl;
  std::cout << "-I- " << myName << ": Starting run" << std::endl;
  run->Run(0, nEvents);
  // ------------------------------------------------------------------------

  // -----   Finish   -------------------------------------------------------
  timer.Stop();


  FairMonitor::GetMonitor()->Print();
  Double_t rtime = timer.RealTime();
  Double_t ctime = timer.CpuTime();
  std::cout << std::endl << std::endl;
  std::cout << "Macro finished successfully." << std::endl;
  std::cout << "Output file is " << sinkFile << std::endl;
  std::cout << "Parameter file is " << parFile << std::endl;
  std::cout << "Real time " << rtime << " s, CPU time " << ctime << " s" << std::endl;
  std::cout << std::endl;
  std::cout << " Test passed" << std::endl;
  std::cout << " All ok " << std::endl;
  // ------------------------------------------------------------------------

  // -----   Resource monitoring   ------------------------------------------
  // Extract the maximal used memory an add is as Dart measurement
  // This line is filtered by CTest and the value send to CDash
  FairSystemInfo sysInfo;
  Float_t maxMemory = sysInfo.GetMaxMemory();
  std::cout << "<DartMeasurement name=\"MaxMemory\" type=\"numeric/double\">";
  std::cout << maxMemory;
  std::cout << "</DartMeasurement>" << std::endl;

  Float_t cpuUsage = ctime / rtime;
  std::cout << "<DartMeasurement name=\"CpuLoad\" type=\"numeric/double\">";
  std::cout << cpuUsage;
  std::cout << "</DartMeasurement>" << std::endl;
  // ------------------------------------------------------------------------

  // -----   Function needed for CTest runtime dependency   -----------------
  RemoveGeoManager();
  // ------------------------------------------------------------------------
}
